from tinydb import TinyDB
from datetime import datetime

"""
Local Json database for the web server.
Opted for NoSQL solution since it provides more flexibility.

An event is stored when a hog is detected by the system.
The event includes:
- Local timestamp
- The entry point
- Path to a picture of the most recent foot print
"""


class EventObject:
    def __init__(self, id=-1, timestamp: str = datetime.now().strftime("%m/%d/%Y, %H:%M:%S"), entry_point: str = 'A', picture_path: str = f'{datetime.now().strftime("%m/%d/%Y, %H:%M:%S")}.jpeg', weight: float = 0) -> None:
        self.id: int = id
        self.timestamp: str = timestamp
        self.entry_point: str = entry_point
        self.picture_path: str = picture_path
        self.weight: float = weight

    def to_dict(self) -> dict[str, any]:
        return {'id': self.id, 'timestamp': f"{self.timestamp}", 'entry_point': self.entry_point, 'picture_path': self.picture_path, 'weight': self.weight}

    @staticmethod
    def from_dict(document: dict) -> "EventObject":
        return EventObject(id=document["id"], timestamp=document["timestamp"], entry_point=document["entry_point"], picture_path=document["picture_path"], weight=document["weight"])

    def set_id(self, id: int):
        self.id = id

    def get_id(self) -> int:
        return self.id


class WebDB:

    def __init__(self, db_name="./db.json") -> None:
        """
        Initialize the databse, if a path to the database is not provided it will create/default to "db.json" in the current folder.
        """
        self.db: TinyDB = TinyDB(db_name)

    def insert(self, event: EventObject):
        id: int = self.db.insert(event.to_dict())
        event.set_id(id=id)
        self.db.update({'id': id}, doc_ids=[id])

    def update(self, event: EventObject):
        self.db.update(event.to_dict(), doc_ids=[event.get_id()])

    def remove(self, event: EventObject):
        self.db.remove(doc_ids=[event.get_id()])

    def get(self, id: int) -> EventObject:
        document = self.db.get(doc_id=id)
        if (document):
            return EventObject.from_dict(document=self.db.get(doc_id=id))
        return None
